--[[
Copyright 2016 MoikMellah

This file is part of the NoName sample module for use with the SYPHA Engine.

The NoName module is free software: you can redistribute it and/or modify it under the terms
of the CC-By-SA 4.0 license as published by Creative Commons.

This code is provided strictly as-in and WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the included file 'LICENSE.CC-BY-SA.txt' for full details.

You should have received a copy of the CC-By-SA 4.0 License along with
the NoName module. If not, see https://creativecommons.org/licenses/by-sa/4.0/
]]--

function Actor:fetchItemAction(i_item, position)

	local it = i_item or 1
	local pos = position or 'stand'

	local itemId = 'PUNCH'

	self.inv = self.inv or {}
	self.inv.item = self.inv.item or {}

	local myItem = self.inv.item[it]

	if(type(myItem) == 'table') then
		itemId = myItem.itemId or 'PUNCH'
	end

	local item = self.scene.env.itemLib.items[itemId]
	
	if(type(item) == 'table') then
		if(item.class == 'melee') then
			if(string.find(item.attrs, 'wpnheavy') ~= nil) then
				self.vars.atkWeightMult = gconf.heavyWpnSpeed or 2
			elseif(string.find(item.attrs, 'wpnmedium') ~= nil) then
				self.vars.atkWeightMult = gconf.mediumWpnSpeed or 1.5
			else
				self.vars.atkWeightMult = nil
			end
		else
			self.vars.atkWeightMult = nil
		end
	end
--[[
	self.vars.nxHelper = tableCopy(item.helperTemplate)
	
	local nx = self.vars.nxHelper

	nx.attrs = item.attrs
	nx.attack = tableCopy(item.attack)

	if(type(myItem) == 'table' and type(myItem.mod) == 'table') then
	
		for k = 1,3 do
		
			local mid = myItem.mod[k]
			
			local m = self.scene.env.modLib.mods[mid]
			
			if(type(m) == 'table') then
				nx.attack = nx.attack or {}
				nx.attack.dmg = (nx.attack.dmg or 0) * (1 + (m.mult or 0))
				if(type(m.attrs) == 'string') then nx.attrs = (nx.attrs or '') .. ',' .. m.attrs end
				if(type(m.attack) == 'table') then
					if(type(m.attack.buff) == 'table') then
						nx.attack.buff = tableCopy(m.attack.buff)
					end
					for k,v in pairs(m.attack) do
						if(type(v) == 'number') then
							nx.attack[k] = (nx.attack[k] or 0) + v
						end
					end
				end
			end
			
		end
	
	end
	
	local t_attrs = strToTable(nx.attrs or '', ',')
			
	local atkmult = 1
	
	local attrdone = {}
	
	for ak,av in ipairs(t_attrs) do
		if(attrdone[av] == nil) then atkmult = atkmult + (self.current['dmg' .. av] or 0) attrdone[av] = true end
	end
	
	nx.attack.dmg = (nx.attack.dmg or 0) * atkmult
]]--
	self.vars.nxHelper = self:assembleItemTemplate(i_item, true)
	
	return item.atkAction[pos]

end

function Actor:assembleDropTemplate(s_item, b_randomizeMods, i_lvl)

	if(s_item == nil) then return nil end
	
	local item = self.scene.env.itemLib.items[s_item]
	
	if(item == nil) then return nil end
	
	local retItem = tableCopy(item.pickupTemplate)
	
	retItem.itemId = s_item
	
	if(b_randomizeMods) then

		retItem.mod = self:getRandomMods(item.class,i_lvl)

	end
	
	return retItem

end

function Actor:getRandomMods(class,lvl)

	local retMods = {}

	for k = 1,3 do

		if(love.math.random() > .3) then
		
			retMods[k] = self.scene.env.modLib:getRandomMod(class, k, lvl)
		
		end

	end
	
	return retMods

end

function Actor:pickupItem(that)

	if(that == nil) then return nil end

	self.inv = self.inv or {}
	
	self.inv.item = self.inv.item or {}
	
	local p = nil
	
	for k = 14,61 do
		if(self.inv.item[k] == nil) then
			p = k
			break
		end
	end
	
	if(p ~= nil) then
		self.inv.item[p] = { itemId = that.itemId, level = that.vars.level }
		if(that.mod ~= nil) then
			self.inv.item[p].mod = tableCopy(that.mod)
		end
		self.scene:remove(that)
		
		local tItem = self:assembleItemTemplate(p)
		
		local lbl = self:spawn('introLabel', 16, 15, 10)
		lbl.camera = 2
		lbl.prim[1].text = tItem.displayName
		lbl.prim[1].bkgColor.r = .8 
		lbl.prim[1].bkgColor.b = 0

	end

end

function Actor:assembleItemTemplate(i_item, withDefault)

	local it = i_item or 1

	local itemId = 'PUNCH'

	self.inv = self.inv or {}
	self.inv.item = self.inv.item or {}

	local myItem = self.inv.item[it]

	local itemLvl = 1

	if(type(myItem) == 'table') then
		itemId = myItem.itemId or 'PUNCH'
		itemLvl = myItem.level or 1
	elseif withDefault == true then
		itemId = 'PUNCH'
	else
		return nil
	end
	
	local statBase = Actor:getStatMultiplier(itemLvl)

	local item = self.scene.env.itemLib.items[itemId]
	local prefix = ''
	local suffix = ''
	local cleanAttrs = 'Attrs: '

	local nx = tableCopy(item.helperTemplate)

	nx.id = item.id
	nx.attrs = item.attrs
	nx.attack = tableCopy(item.attack) or {}
	nx.equip = tableCopy(item.equip) or {}

	nx.stats = {}

	nx.shortName = item.displayName or 'Thing'

	--nx.displayName = item.displayName or 'Thing'
	
	nx.descr = item.descr

	if(type(myItem) == 'table' and type(myItem.mod) == 'table') then
		for k = 1,3 do
			local mid = myItem.mod[k]
			local m = self.scene.env.modLib.mods[mid]
			
			if(type(m) == 'table') then
			
				if(m.descr ~= nil) then nx.descr = nx.descr .. ' ' .. m.descr end
			
				if(k == 3) then
					suffix = ' ' .. m.title
				else
					prefix = prefix .. m.title .. ' '
				end
			
				nx.attack = nx.attack or {}
				nx.attack.dmg = (nx.attack.dmg or 0) * (1 + (m.mult or 0))
				nx.equip.def = (nx.equip.def or 0) * (1 + (m.mult or 0))
				nx.equip.mdef = (nx.equip.mdef or 0) * (1 + (m.mult or 0))
				
				if(type(m.attrs) == 'string') then nx.attrs = (nx.attrs or '') .. ',' .. m.attrs end
				
				if(type(m.attack) == 'table') then
					if(type(m.attack.buff) == 'table') then
						nx.attack.buff = nx.attack.buff or {}
						for mbk,mbv in pairs(m.attack.buff) do
							if(type(mbv) == 'table') then 
								local mbf = tableCopy(mbv)
								mbf.name = m.id .. '-' .. mbk
								table.insert(nx.attack.buff, mbf)
							end
						end
						--nx.attack.buff = tableCopy(m.attack.buff)
					end
					for k,v in pairs(m.attack) do
						if(type(v) == 'number') then
							nx.attack[k] = (nx.attack[k] or 0) + v
						end
					end
				end
				if(type(m.equip) == 'table' and type(m.equip.buff) == 'table') then
					for bk,bv in pairs(m.equip.buff) do
						for ek,ev in pairs(bv) do
							if(type(ev) == 'number') then
								local lvmult = statBase
								if(bv.flat == 1) then lvmult = 1 end
								nx.stats[ek] = math.ceil((nx.stats[ek] or 0) + (ev * lvmult))
							end
						end
					end
				end
			end
			
		end
	
	end
	
	local t_attrs = strToTable(nx.attrs or '', ',')
			
	local atkmult = 1
	local defmult = 1
	
	local attrdone = {}
	
	local amore = false
	
	for ak,av in ipairs(t_attrs) do
		if(attrdone[av] == nil) then 
			atkmult = atkmult + (self.current['dmg' .. av] or 0)
			defmult = defmult + (self.current['def' .. av] or 0)
			
			local ca = string.gsub(av, '^wpn', '')
			ca = string.gsub(ca, '^def', '')
			
			if(amore) then
				cleanAttrs = cleanAttrs .. ','
			else
				amore = true
			end
			
			cleanAttrs = cleanAttrs .. string.upper(ca)
						
			attrdone[av] = true 
		end
	end
	
	nx.attack.dmg = (nx.attack.dmg or 0) * atkmult * statBase
	nx.equip.def = (nx.equip.def or 0) * defmult * statBase
	nx.equip.mdef = (nx.equip.mdef or 0) * defmult * statBase

	nx.stats.id = nil
	if(nx.attack.dmg > 0) then nx.stats.atk = math.ceil(nx.attack.dmg) end
	if(nx.equip.def > 0) then nx.stats.def = math.ceil(nx.equip.def) end
	if(nx.equip.mdef > 0) then nx.stats.mdef = math.ceil(nx.equip.mdef) end
	
	local more = false
	local statStr = ''
	
	for kk,kv in pairs(nx.stats) do
		if more then
			statStr = statStr .. ', '
		else
			more = true
		end
		
		statStr = statStr .. string.upper(kk) .. ' '
		if(type(kv) == 'number' and kv >= 0) then
			statStr = statStr .. '+'
		end
		statStr = statStr .. kv
		
	end
	
	nx.displayName = 'Lv ' .. itemLvl .. ' ' .. prefix .. nx.shortName .. suffix
	
	nx.fulltext = nx.displayName
	nx.fulltext = nx.fulltext .. '\n' .. (nx.descr or '')
	nx.fulltext = nx.fulltext .. '\n' .. (statStr or '')
	nx.fulltext = nx.fulltext .. '\n' .. (cleanAttrs or '')
	
--	--debugPrint(nx)
	
	return nx

end

function Actor:equipmentUiNav(s_btn)

	local btn = s_btn or 'right'
	
	local nbtn = string.sub(btn,1,1)

	self.vars.posId = self.vars.posId or 1

	local oldId = self.vars.posId

--[[
	local op = self.vars.pos[self.vars.posId] or {}
	self.vars.posId = op[nbtn] or self.vars.posId
	local np = self.vars.pos[self.vars.posId]
	self.prim[1].x = np.x
	self.prim[1].y = np.y
	self.prim[1].w = np.w or 1
	self.prim[1].h = np.h or 1
]]--

	self.scene.vars.guiArray = self.scene.vars.guiArray or {}

	local op = self.scene.vars.guiArray[self.vars.posId]

	if(op == nil) then return nil end
	
	self.vars.posId = op[nbtn] or self.vars.posId

	local np = self.scene.vars.guiArray[self.vars.posId]
	
	op.prim[4].visible = 0
	np.prim[4].visible = 1

	local hero = self.scene.vars.sourceActor
	
	if(hero ~= nil) then
		local nx = hero:assembleItemTemplate(self.vars.posId)
		if(nx ~= nil) then
			self.prim[9].text = nx.fulltext
		else
			self.prim[9].text = 'Empty'
		end
	end

end

function Actor:equipmentClick()

	if(self.vars.selected == nil) then
		
		local op = self.scene.vars.guiArray[self.vars.posId]
		
		if(op == nil) then return nil end

		self.vars.selected = self.vars.posId
		
		op.prim[1].visible = 1
		
	else
	
		local hero = self.scene.vars.sourceActor
	
		local i = hero.inv.item
		
		if(self:canSwapItems(self.vars.posId, self.vars.selected)) then
			i[self.vars.posId],i[self.vars.selected] = i[self.vars.selected],i[self.vars.posId]
		end
		
		self.scene.vars.guiArray[self.vars.selected].prim[1].visible = 0
		
		self.vars.selected = nil

		hero:setEquipmentBuffs()
		hero:rollStats()

		self:getHeroStats()		
		self:equipmentUiNav('none')
		
	end

end

function Actor:canSwapItems(fromPos, toPos)

	local posA = fromPos or 1
	local posB = toPos or 1
	
	local hero = self.scene.vars.sourceActor
	hero.inv = hero.inv or {}
	hero.inv.item = hero.inv.item or {}
	
	local itemIdA = 'none'
	local itemTmpA = {}
	local itemIdB = 'none'
	local itemTmpB = {}
	
	if(hero.inv.item[posA] ~= nil) then
		itemIdA = hero.inv.item[posA].itemId
		itemTmpA = self.scene.env.itemLib.items[itemIdA]
	end
	if(hero.inv.item[posB] ~= nil) then
		itemIdB = hero.inv.item[posB].itemId
		itemTmpB = self.scene.env.itemLib.items[itemIdB]
	end
	
	local slotA = self.scene.vars.guiArray[posA]
	local slotB = self.scene.vars.guiArray[posB]
	
	if((itemIdA ~= 'none') and (string.find(itemTmpA.eq, slotB.slot) == nil) and (slotB.slot ~= 'inv')) then
		return false
	end
	if((itemIdB ~= 'none') and (string.find(itemTmpB.eq, slotA.slot) == nil) and (slotA.slot ~= 'inv')) then
		return false
	end
	
	return true

end

function Actor:getHeroStats()

	local hero = self.scene.vars.sourceActor
	
	if(hero == nil) then return nil end

	local str = "\n\n"
	--str = str + "HP\nMP\nSTR\nCON\nINT\nWIS\nAGI\nLCK"
	str = str .. math.ceil(hero.current.hp) .. '/' .. math.ceil(hero.current.maxhp) .. '\n'
	str = str .. math.ceil(hero.current.mp) .. '/' .. math.ceil(hero.current.maxmp) .. '\n\n'
	str = str .. math.ceil(hero.current.def) .. '\n'
	str = str .. math.ceil(hero.current.mdef) .. '\n\n'
	str = str .. math.ceil(hero.current.str) .. '\n'
	str = str .. math.ceil(hero.current.con) .. '\n'
	str = str .. math.ceil(hero.current.int) .. '\n'
	str = str .. math.ceil(hero.current.wis) .. '\n'
	str = str .. math.ceil(hero.current.agi) .. '\n'
	str = str .. math.ceil(hero.current.lck) .. '\n\n'
	
	str = str .. hero.current.level .. '\n'
	str = str .. hero.base.xp .. '\n'
	str = str .. ((gconf.expPerLvl or 3000) * hero.current.level) .. '\n\n'

	self.prim[8].text = str

end

function Actor:invPopulateItems()

	local hero = self.scene.vars.sourceActor
	
	if(hero == nil) then return nil end

	hero.inv = hero.inv or {}

	local primStart = self.vars.primStart or #self.prim + 1
	
	for k = 1,61 do

		local p = k + primStart - 1

		local pos = self.vars.pos[k]
		
		self.prim[p] = self.prim[p] or { type = 'image', visible = 0, x = pos.x, y = pos.y }
		
		if(hero.inv.item[k] ~= nil) then
			local item = hero.inv.item[k].itemId
			local itemTmp = self.scene.env.itemLib.items[item].pickupTemplate
			self.prim[p].visible = 1
			self.prim[p].imageSet = itemTmp.imageSet
			self.prim[p].quad = 'ICON-1'
			--debugPrint('Creating item image #'..k..' on prim #' .. p .. ' w/imgSet '..itemTmp.imageSet)		
		else
			self.prim[p].visible = 0
		end
		
	end

end

function Actor:setSingleInvIcon()

	local hero = self.scene.hero or self.scene.env.vars.hero
	
	if(hero == nil) then return nil end

	local k = self.invPos or 0

	if(self.slot ~= nil) then
		self.prim[2].visible = 1
	end

	if(hero.inv.item[k] ~= nil) then
		local item = hero.inv.item[k].itemId
		local itemTmp = self.scene.env.itemLib.items[item].pickupTemplate
		self.prim[3].visible = 1
		self.prim[3].imageSet = itemTmp.imageSet
		self.prim[3].quad = 'ICON-2'
		self.prim[2].quad = 'empty'
	else
		self.prim[3].visible = 0
		if(self.slot ~= 'inv') then
			self.prim[2].quad = self.slot
		else
			self.prim[2].quad = 'empty'
		end
	end

end

function Actor:getRandomItem(class, opener, maxLv)

	local ml = maxLv or 1
	
	local ic = class or 'melee'
	
	local o = opener or self.scene.hero or self.scene.env.vars.hero
	
	local item = Queryable.new(self.scene.env.itemLib.items
		):Where('x => (x.pickupTemplate ~= nil) and (x.class == "'..ic..'") and ((x.minLevel or 999) <= '..ml..') and ((x.maxLevel or 999) >= '.. ml ..')'
		):Select('x => {id = x.id, sort = love.math.random()}'
		):OrderBy('x => x.sort'
		):First()
		
	return item.id

end
